package es.um.atica.miproyecto.security.authentication;

import static org.jboss.seam.annotations.Install.FRAMEWORK;

import javax.crypto.Cipher;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.SecretKeySpec;
import javax.ejb.Stateless;

import org.jboss.seam.Component;
import org.jboss.seam.annotations.Install;
import org.jboss.seam.annotations.Logger;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.log.Log;

import com.sun.jersey.core.util.Base64;

import es.um.atica.enclave2.security.authorization.services.ServicioAutorizacionRest;
import es.um.atica.enclave2.security.rest.authorization.entities.RestUsuarios;

@Name( "restAuthenticationProvider" )
@Install(
    precedence = FRAMEWORK )
@Stateless
public class RestAuthenticationProvider {

    @Logger
    Log log;

    // TODO: ESPECIFICAR EN FILTO 
    byte CLAVE_ENCRIPTACION[] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
    byte VECTOR_INICIALIZADOR[] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
    
    private ServicioAutorizacionRest servicioAutorizacionRest;

    /**
     * Autentica al usuario de un servicio rest en base al par usuario/contrasenia
     * 
     * @param user
     *            Usuario con el que hacer login en el servicio
     * @param password
     *            Password del usuario en el servicio
     * @return el token del usuario, si este pasa la autenticacion o null en caso de que no la pase
     */
    public String authenticateUser( String user, String password ) {
        RestUsuarios usuario = getServicioAutorizacion().obtenerUsuario( user );
        if ( usuario == null ) {
            return null;
        }

        String passwordUsuarioDesdencriptado = this.desencriptaPassword( usuario.getPassword() );
        if ( passwordUsuarioDesdencriptado.equals( password ) ) {
            return getServicioAutorizacion().generaTokenUsuario( usuario );
        }
        else {
            return null;
        }
    }

    /**
     * Autentica el token de usuario basado en el par usuario/token y refresca el tiempo de vida del token en base de
     * datos
     * 
     * @param user
     *            Usuario para el que se valida el token
     * @param token
     *            Token a validar
     * @return devuelve true si el par Usuario/token se encuentra en la base de datos y no esta caducado, false en
     *         cualquier otro caso
     */
    public boolean authenticanteToken( String user, String token ) {
        RestUsuarios usuario = getServicioAutorizacion().obtenerUsuario( user );
        if ( usuario == null ) {
            return false;
        }
        return getServicioAutorizacion().autenticaTokenUsuario( usuario, token );
    }

    public String encriptaPassword( String passwordEnClaro ) {
        try {
            IvParameterSpec iv = new IvParameterSpec( VECTOR_INICIALIZADOR );
            SecretKeySpec skeySpec = new SecretKeySpec( CLAVE_ENCRIPTACION, "AES" );

            Cipher cipher = Cipher.getInstance( "AES/CBC/PKCS5PADDING" );
            cipher.init( Cipher.ENCRYPT_MODE, skeySpec, iv );

            byte[] encrypted = cipher.doFinal( passwordEnClaro.getBytes() );

            return new String( Base64.encode( encrypted ) );
        }
        catch ( Exception e ) {
            log.error( "Error encriptando password", e );
            return null;
        }
    }

    public String desencriptaPassword( String passwordEncriptado ) {
        try {
            IvParameterSpec iv = new IvParameterSpec( VECTOR_INICIALIZADOR );
            SecretKeySpec skeySpec = new SecretKeySpec( CLAVE_ENCRIPTACION, "AES" );

            Cipher cipher = Cipher.getInstance( "AES/CBC/PKCS5PADDING" );
            cipher.init( Cipher.DECRYPT_MODE, skeySpec, iv );

            byte[] original = cipher.doFinal( Base64.decode( passwordEncriptado ) );

            return new String( original );
        }
        catch ( Exception e ) {
            log.error( "Error encriptando password", e );
            return null;

        }

    }

    private ServicioAutorizacionRest getServicioAutorizacion() {
        if(this.servicioAutorizacionRest==null){
            this.servicioAutorizacionRest = ( ServicioAutorizacionRest ) Component.getInstance( "servicioAutorizacionRest", true );
        }
        return this.servicioAutorizacionRest;
    }

}
