package es.um.atica.miproyecto.security.authorization.das.impl;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.UUID;

import javax.ejb.Local;
import javax.ejb.Stateless;
import javax.persistence.Query;

import org.jboss.seam.annotations.Name;

import es.um.atica.commons.utils.UtilDate;
import es.um.atica.enclave2.security.authorization.das.interfaces.RestTokenDataAccessService;
import es.um.atica.enclave2.security.rest.authorization.entities.RestUsuarioToken;
import es.um.atica.enclave2.security.rest.authorization.entities.RestUsuarios;
import es.um.atica.jpa.das.DataAccessServiceImpl;

@Local( RestTokenDataAccessService.class )
@Stateless
@Name( "restTokenDataAccessService" )
public class RestTokenDataAccessServiceImpl extends DataAccessServiceImpl<RestUsuarioToken> implements
        RestTokenDataAccessService {

    public String generaTokenUsuario( RestUsuarios usuario ) {
        boolean tokenCreado = false;
        String token = null;
        HashMap<String, Object> parametros = new HashMap<String, Object>();
        while ( !tokenCreado ) {
            long totalToken = 0;
            token = UUID.randomUUID().toString();
            parametros.put( "token", token );
            // Compruebo que el token no este repetido, en caso contrario geenro uno nuevo
            totalToken = this.countTotalRecordByEntityNamedQuery( "cuentaTodosTokenExistentes",
                                                                  Collections.emptyList(), parametros, "and",
                                                                  Collections.emptyMap() );
            if ( totalToken == 0 ) {
                limpiaTokensDeUsuario( usuario );
                tokenCreado = true;
                RestUsuarioToken tokenParaUsuario = new RestUsuarioToken();
                tokenParaUsuario.setRestUsuarios( usuario );
                tokenParaUsuario.setToken( token );
                tokenParaUsuario.setFechaCreacion( new Date() );
                this.persist( tokenParaUsuario, true );
            }
        }
        return token;
    }

    public void actualizaFechaExpiracionTokenUsuario( RestUsuarioToken token ) {
        token.setFechaCreacion( new Date() );
        this.merge( token,true );
    }

    public RestUsuarioToken obtenerTokenValidoDeUsuario( RestUsuarios usuario ) {
        limpiaTokensCaducados( usuario );      
        if ( usuario.getRestUsuarioTokens() == null || usuario.getRestUsuarioTokens().size() == 0 || usuario.getRestUsuarioTokens().size() > 1 ) {
            return null;
        }
        else {
            RestUsuarioToken token = usuario.getRestUsuarioTokens().get( 0 );
            actualizaFechaExpiracionTokenUsuario( token );
            return token;
        }
    }

    private void limpiaTokensCaducados( RestUsuarios usuario ) {
        List<RestUsuarioToken> tokenABorrar = new ArrayList<RestUsuarioToken>();
        for ( RestUsuarioToken token : usuario.getRestUsuarioTokens() ) {
            long fechaMax = UtilDate.fechaHoy().getTime() - usuario.getTiempoVidaToken();
            if ( token.getFechaCreacion().getTime() < fechaMax ) {
                tokenABorrar.add( token );
            }
        }

        for ( RestUsuarioToken token : tokenABorrar ) {
            usuario.getRestUsuarioTokens().remove( token );
            this.delete( token, true );

        }
        this.getEntityManager().merge( usuario );
        this.getEntityManager().flush();
    }

    private void limpiaTokensDeUsuario( RestUsuarios usuario ) {
        for ( RestUsuarioToken tokenABorrar : usuario.getRestUsuarioTokens() ) {
            this.delete( tokenABorrar, true );
        }
        usuario.setRestUsuarioTokens( new ArrayList<RestUsuarioToken>() );
        this.getEntityManager().merge( usuario );
        this.getEntityManager().flush();
    }
}
