package es.um.atica.miproyecto.security.rest.filter;

import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.Response.ResponseBuilder;

import org.apache.log4j.Logger;
import org.jboss.seam.Component;

import com.sun.jersey.core.util.Base64;
import com.sun.jersey.spi.container.ContainerRequest;
import com.sun.jersey.spi.container.ContainerRequestFilter;

import es.um.atica.enclave2.security.authentication.RestAuthenticationProvider;

public class RestRequestFilter implements ContainerRequestFilter {

    private Logger log = Logger.getLogger( RestRequestFilter.class );
    private final String CABECERA_AUTORIZACION = "Authorization";
    private final String NO_AUTORIZADO = "No autorizado";

    @Override
    /**
     * Filtra la peticion REST y segun la URL accedida si contiene "private" busca el par usuario:token en la 
     * cabecera de autenticacion y valida, si es "public" devuelve la pagina solicitada
     */
    public ContainerRequest filter( ContainerRequest request ) {
        log.debug( "Entro en el filtro" );
        RestAuthenticationProvider provider = this.getAuthenticationProvider();

        String cabeceraAutorizacion = request.getHeaderValue( CABECERA_AUTORIZACION );
        if ( request.getPath().contains( "private/" ) ) {
            // Si no tiene cabecera de autorizacion no autorizamos
            if ( cabeceraAutorizacion == null ) {
                this.denegarConexion();
                return null;
            }
            String[] valoresCabecera = Base64.base64Decode( cabeceraAutorizacion ).split( ":" );
            // Si en la cabecera de autorizacion no van los dos parametros (nombre y token) no autorizamos
            if ( valoresCabecera.length != 2 ) {
                this.denegarConexion();
                return null;
            }
            String usuario = valoresCabecera[0];
            String token = valoresCabecera[1];
            //Si no pasa la autenticacion del token no autorizamos
            if ( !provider.authenticanteToken( usuario, token ) ) {
                this.denegarConexion();
                return null;
            }
            else {
                // Ha pasado la autenticacion, por lo que se lanza la peticion
                return request;
            }
        }
        else {
            //Es una peticion normal que no va por /private por lo que se lanza sin validar
            return request;
        }

    }

    /**
     *  Termina la peticion entrante contestando con no autorizado
     */
    private void denegarConexion() {
        ResponseBuilder builder = null;
        builder = Response.status( Response.Status.UNAUTHORIZED ).entity( NO_AUTORIZADO );
        throw new WebApplicationException( builder.build() );
    }

    private RestAuthenticationProvider getAuthenticationProvider() {
        return ( RestAuthenticationProvider ) Component.getInstance( "restAuthenticationProvider", true );
    }

}
