package es.um.atica.miproyecto.security.rest.filter;

import javax.ws.rs.core.Response;
import javax.ws.rs.core.Response.ResponseBuilder;

import org.apache.log4j.Logger;
import org.jboss.seam.Component;

import com.sun.jersey.core.util.Base64;
import com.sun.jersey.spi.container.ContainerRequest;
import com.sun.jersey.spi.container.ContainerResponse;
import com.sun.jersey.spi.container.ContainerResponseFilter;

import es.um.atica.enclave2.security.authentication.RestAuthenticationProvider;

public class RestResponseFilter implements ContainerResponseFilter {

    private Logger log = Logger.getLogger( RestResponseFilter.class );
    private final String CABECERA_AUTORIZACION = "Authorization";
    private final String LOGIN_INCORRECTO = "Login no válido";

    private RestAuthenticationProvider restAuthenticationProvider;

    @Override
    /**
     * Filtra la respuesta a la peticion REST
     * Tratamos e lcaso de public/login
     */
    public ContainerResponse filter( ContainerRequest request, ContainerResponse response ) {
        // SOLO trato el caso de NOT_FOUND con login
        String cabeceraAutorizacion = request.getHeaderValue( CABECERA_AUTORIZACION );
        RestAuthenticationProvider provider = this.getAuthenticationProvider();

        if ( request.getPath().endsWith( "public/login" ) ) {
            log.info( "Llamada a login servicio REST" );
            // Si no tiene cabecera de autorización deniego la peticion
            if ( cabeceraAutorizacion == null ) {
                log.info( "No se envió la cabecera de autenticacion" );
                response.setResponse( denegarConexion() );
                return response;
            }
            String[] valoresCabecera = Base64.base64Decode( cabeceraAutorizacion ).split( ":" );
            // Compruebo que la cabecera tenga los dos parámetros (usuario:pass)
            if ( valoresCabecera.length != 2 ) {
                log.info( "La cabecera de autenticacion no tiene los parámtros adecuados, requeridos 2, encontrados "+valoresCabecera.length );
                response.setResponse( denegarConexion() );
                return response;
            }
            log.info( "Autenticando a usuario: "+valoresCabecera[0] );
            // Con cabecera de autorizacion y se ha solicitado la url de login, se hace login
            String token = provider.authenticateUser( valoresCabecera[0], valoresCabecera[1] );
            if ( token == null ) {
                log.info( "Autenticancion fallida para usuario: "+valoresCabecera[0] );
                response.setResponse( denegarConexion() );
                return response;
            }
            else {
                log.info( "Usuario "+valoresCabecera[0]+" autenticado con exito, enviado token" );
                response.setResponse( respuestaConToken( new String( Base64.encode( token ) ) ) );
                return response;
            }
        }

        return response;
    }

    /**
     * Genera una respuesta 200 con el token que se ha obtenido
     * 
     * @param token
     * @return
     */
    private Response respuestaConToken( String token ) {
        ResponseBuilder builder = null;
        builder = Response.ok( token );
        return builder.build();
    }

    /**
     * Genera una respuesta de usuario no autorizado
     * 
     * @return
     */
    private Response denegarConexion() {
        ResponseBuilder builder = null;
        builder = Response.status( Response.Status.UNAUTHORIZED ).entity( LOGIN_INCORRECTO );
        return builder.build();
    }

    private RestAuthenticationProvider getAuthenticationProvider() {
        if ( this.restAuthenticationProvider == null ) {
            this.restAuthenticationProvider = ( RestAuthenticationProvider ) Component.getInstance( "restAuthenticationProvider", true );
        }
        return this.restAuthenticationProvider;
    }
}
